/*
** Copyright (C) 2005 CHARY Corentin <iksaif@gmail.com>
**
** Ce programme est libre, vous pouvez le redistribuer et/ou le modifier 
** selon les termes de la Licence Publique Gnrale GNU publie par la Free Software Foundation 
**
** Ce programme est distribu car potentiellement utile, mais SANS AUCUNE GARANTIE, ni explicite ni implicite, 
** y compris les garanties de commercialisation ou d'adaptation dans un but spcifique.
** Reportez-vous  la Licence Publique Gnrale GNU pour plus de dtails.
**
** Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en mme temps que ce programme 
**
** balls.c
**
** Gestion des balles (et des touches pour les dplacer)
**
**
*/

# include "balls.h"

/*
** Initialisation
*/
void InitBalls()
{
	unsigned int i, j;

	/* Alocation de la mmoire */
	balls = malloc(options.playernb * sizeof(struct s_ball));

	/* Pour chaque joueur */
	for (i = 0; i < options.playernb; i++) {
		if (options.playernb - (i + 1) < options.ianb)	/* Les IA sont mises  la fin */
			balls[i].type = BALL_IA;
		else
			balls[i].type = BALL_PLAYER;

		balls[i].speed = options.speed;
		balls[i].accel = BALL_ACCEL;
		balls[i].lifes = BALL_LIFES;
		balls[i].score = 0;
		balls[i].x = myrand(0, SCREEN_WIDTH - BALL_WIDTH);	/* Postionionment different suivant le joueur */
		balls[i].y = myrand(0, SCREEN_HEIGHT - BALL_HEIGHT);
		balls[i].dx = balls[i].dy = 0;
		balls[i].keysdown = 0;
		balls[i].invincibletime = balls[i].invincibletotaltime =
		    balls[i].invinciblelast = 0;
		for (j = 0; j < BONUS_NB; ++j)
			balls[i].bonuscount[j] = 0;
	}
}

/*
** Libration de la mmoire
*/
void FreeBalls()
{
	free(balls);
}

/*
** Dplacements
*/
void MoveBalls()
{
	unsigned int i, j;

	for (i = 0; i < options.playernb; i++) {
		if (balls[i].lifes <= 0)
			continue;

		/* Deplacement horizontal */
		if ((balls[i].keysdown & BALLK_UP)
		    && !(balls[i].keysdown & BALLK_DOWN)
		    && (-balls[i].dy) < balls[i].speed) {
			balls[i].dy -= balls[i].speed / balls[i].accel;	/* Acceleration vers le bas */
		} else if (!(balls[i].keysdown & BALLK_UP)
			   && (balls[i].keysdown & BALLK_DOWN)
			   && balls[i].dy < balls[i].speed) {
			balls[i].dy += balls[i].speed / balls[i].accel;	/* Vers le haut */
		} else if (balls[i].dy && !(balls[i].keysdown & BALLK_UP)
			   && !(balls[i].keysdown & BALLK_DOWN)) {
			balls[i].dy -= balls[i].dy / balls[i].accel;	/* Dceleration */
		}

		/* Dplacement vertical */
		if ((balls[i].keysdown & BALLK_LEFT)
		    && !(balls[i].keysdown & BALLK_RIGHT)
		    && (-balls[i].dx) < balls[i].speed) {
			balls[i].dx -= balls[i].speed / balls[i].accel;	/* vers la gauche */
		} else if (!(balls[i].keysdown & BALLK_LEFT)
			   && (balls[i].keysdown & BALLK_RIGHT)
			   && balls[i].dx < balls[i].speed) {
			balls[i].dx += balls[i].speed / balls[i].accel;	/* vers la droite */
		} else if (balls[i].dx && !(balls[i].keysdown & BALLK_LEFT)
			   && !(balls[i].keysdown & BALLK_RIGHT)) {
			balls[i].dx -= balls[i].dx / balls[i].accel;	/* Dceleration */
		}

		/* verifications et calcul des collisions (si option active) *//* FIXME c'est encore pas top */
		if (modes[options.mode].flags & MODE_ARCADE) {
			double tmpdx[options.playernb];
			double tmpdy[options.playernb];

			for (j = 0; j < options.playernb; j++) {
				tmpdx[j] = balls[j].dx;
				tmpdy[j] = balls[j].dy;
			}

			for (j = 0; j < options.playernb; j++) {
				/* La balle ne peux pas se rencontrer elle mme */
				if (balls[j].lifes > 0 && j != i) {
					/* Si la distance entre les deux balles est plus petite que deux fois le rayon */
					if (COLLBALLS
					    (balls[i].x, balls[i].y,
					     balls[j].x, balls[j].y)) {
						/* sur un bord */
						if (balls[j].x <= 0
						    || balls[j].x >=
						    SCREEN_WIDTH -
						    BALL_WIDTH)
							tmpdx[i] *= -1;
						else
							tmpdx[i] +=
							    balls[j].dx *
							    2;

						/* sur un bord */
						if (balls[j].y <= 0
						    || balls[j].y >=
						    SCREEN_HEIGHT -
						    BALL_HEIGHT)
							tmpdy[i] *= -1;
						else
							tmpdy[i] +=
							    balls[j].dy *
							    2;

					}

				}
			}

			for (j = 0; j < options.playernb; j++) {
				balls[j].dx = tmpdx[j];
				balls[j].dy = tmpdy[j];
			}
		}

		/* Ajout des differentiels de dplacements */
		balls[i].y += balls[i].dy;
		if (balls[i].y < 0)
			balls[i].y = 0;
		if (balls[i].y > SCREEN_HEIGHT - BALL_HEIGHT)
			balls[i].y = SCREEN_HEIGHT - BALL_HEIGHT;

		balls[i].x += balls[i].dx;
		if (balls[i].x < 0)
			balls[i].x = 0;
		if (balls[i].x > SCREEN_WIDTH - BALL_WIDTH)
			balls[i].x = SCREEN_WIDTH - BALL_WIDTH;

	}
}

/*
** Calculs de L'IA
*/
void IAMovesBalls(struct s_bonus *bonus, struct s_walls *walls)
{
	unsigned int i, j;
	unsigned int nowalls;
	int diffx, diffy;
	double maxscore, score;
	struct s_1bonus *pbonus;

	for (i = options.playernb - options.ianb; i < options.playernb; i++) {	/* Pour chaque balles */
		diffx = diffy = 0;
		maxscore = 0;
		nowalls = 1;

		/* En Premier, on vite les murs */
		/* On vite le mus le plus proche, et seulement si il est a moins de 1 fois la taille du mur + 1 fois la taille de la balle */
		for (j = 0;
		     j < walls->wallsnb
		     && balls[i].invincibletotaltime == 0 && nowalls;
		     j++) {


			if (walls->walls[j].type == WALLS_HORI) {
				diffx =
				    (balls[i].x + BALL_WIDTH / 2) -
				    (walls->walls[j].x +
				     (options.wallsize * WALLS_SIZE_COEF) /
				     2);
				diffy =
				    (balls[i].y + BALL_WIDTH / 2) -
				    (walls->walls[j].y + WALLS_WIDTH);

				if (abs(diffy) <
				    (BALL_WIDTH + WALLS_WIDTH) * 2
				    && abs(diffx) <
				    (BALL_WIDTH +
				     options.wallsize * WALLS_SIZE_COEF) *
				    1.5) {
					diffy *= -0.5;
					diffx *= -0.5;
					nowalls = 0;
				}
			} else {
				diffx =
				    (balls[i].x + BALL_WIDTH / 2) -
				    (walls->walls[j].x + WALLS_WIDTH);
				diffy =
				    (balls[i].y + BALL_WIDTH / 2) -
				    (walls->walls[j].y +
				     (options.wallsize * WALLS_SIZE_COEF) /
				     2);

				if (abs(diffx) <
				    (BALL_WIDTH + WALLS_WIDTH) * 2
				    && abs(diffy) <
				    (BALL_WIDTH +
				     options.wallsize * WALLS_SIZE_COEF) *
				    1.5) {
					diffy *= -0.5;
					diffx *= -0.5;
					nowalls = 0;
				}
			}
		}

		/* Si y'en a pas de dangereux, on va vers le bonus le mieux ... */
		pbonus = bonus->first;
		while (pbonus && nowalls) {

			score = (double) bonusconf[pbonus->type].pts * 5 +
			    (100 -
			     (double) bonusconf[pbonus->type].proba) * 2 +
			    (double) count_ones(bonusconf[pbonus->type].
						flags) * 2 -
			    ((double) abs(balls[i].x - pbonus->x) +
			     (double) abs(balls[i].y - pbonus->y));

			if (maxscore == 0 || score > maxscore) {
				maxscore = score;
				diffx =
				    (balls[i].x - pbonus->x) * myrand(90,
								      110)
				    / 100;
				diffy =
				    (balls[i].y - pbonus->y) * myrand(90,
								      110)
				    / 100;
			}

			pbonus = pbonus->next;
		}

		balls[i].keysdown = 0;

		if (diffy < 0)
			balls[i].keysdown |= BALLK_DOWN;
		else if (diffy > 0)
			balls[i].keysdown |= BALLK_UP;

		if (diffx < 0)
			balls[i].keysdown |= BALLK_RIGHT;
		else if (diffx > 0)
			balls[i].keysdown |= BALLK_LEFT;

	}
}

/*
** Gestion des balles 
*/
void HandleBalls(struct s_bonus *bonus, struct s_walls *walls)
{
	SDL_Rect dst, src;
	unsigned int now;
	unsigned int i;

	IAMovesBalls(bonus, walls);	/* Calcul des touches que va activer l'IA */
	MoveBalls();

	/* Cette boucle sera surement regoupe plus tard avec d'autres du mme genre */
	now = SDL_GetTicks();
	for (i = 0; i < options.playernb; i++) {
		if (balls[i].lifes <= 0)
			continue;

		/* Diminution du temps restant en mode invincible */
		if (balls[i].invincibletime > 0) {
			if (now - balls[i].invinciblelast < balls[i].invincibletime) {	/* Si il en reste, on le diminue */
				balls[i].invincibletime -=
				    now - balls[i].invinciblelast;
				balls[i].invinciblelast = now;
			} else {
				balls[i].invincibletime =
				    balls[i].invinciblelast =
				    balls[i].invincibletotaltime = 0;
			}
		}
	}

	src.x = src.y = 0;
	src.w = BALL_WIDTH;
	src.h = BALL_HEIGHT;

	dst.w = src.w;
	dst.h = src.h;
	/* Pour tous les joueurs */
	for (i = 0; i < options.playernb; i++) {
		if (balls[i].lifes <= 0)
			continue;	/* Si il n'a plus de vie, on le laisse */

		dst.x = balls[i].x;
		dst.y = balls[i].y;

		if (balls[i].invincibletime > 0) {
			if ((balls[i].invincibletime / 100) % 2)	/* une balle invincible clignote */
				SDL_BlitSurface(gameui.balls[i], &src,
						SDL_GetVideoSurface(),
						&dst);
		} else
			SDL_BlitSurface(gameui.balls[i], &src,
					SDL_GetVideoSurface(), &dst);
	}
}

/*
** Gestion des touches de dplacement des balles
*/
void EventBalls(SDL_Event * event)
{
	unsigned int i;

	if (!event)
		return;

	/*
	 ** balls[joueur].keydown permet de stoquer les touches appuyes
	 ** 4 bit sont donc utiliss, 1 par touche
	 ** ces bits sont mis  0 ou a 1 si la touche qui correspond est appuye ou pas
	 */
	switch (event->type) {
	case SDL_KEYDOWN:
		if (event->key.state == SDL_PRESSED) {	/* Touches appuyes */
			/* Pour chaque joueurs, on regarde si la touche appuye correspond a l'une de celles qui est dans leurs options */
			for (i = 0; i < options.playernb - options.ianb;
			     i++) {
				if (event->key.keysym.sym ==
				    options.keys[i][OPTK_UP])
					balls[i].keysdown |= BALLK_UP;
				else if (event->key.keysym.sym ==
					 options.keys[i][OPTK_DOWN])
					balls[i].keysdown |= BALLK_DOWN;
				else if (event->key.keysym.sym ==
					 options.keys[i][OPTK_LEFT])
					balls[i].keysdown |= BALLK_LEFT;
				else if (event->key.keysym.sym ==
					 options.keys[i][OPTK_RIGHT])
					balls[i].keysdown |= BALLK_RIGHT;
			}
		}
		break;
	case SDL_KEYUP:	/* Touches relaches */
		if (event->key.state == SDL_RELEASED) {
			/* Idem, mais pour les touches relaches */
			for (i = 0; i < options.playernb - options.ianb;
			     i++) {
				if (event->key.keysym.sym ==
				    options.keys[i][OPTK_UP])
					balls[i].keysdown ^= BALLK_UP;
				else if (event->key.keysym.sym ==
					 options.keys[i][OPTK_DOWN])
					balls[i].keysdown ^= BALLK_DOWN;
				else if (event->key.keysym.sym ==
					 options.keys[i][OPTK_LEFT])
					balls[i].keysdown ^= BALLK_LEFT;
				else if (event->key.keysym.sym ==
					 options.keys[i][OPTK_RIGHT])
					balls[i].keysdown ^= BALLK_RIGHT;
			}

		}
		break;
	}

}
