/*
** Copyright (C) 2005 CHARY Corentin <iksaif@gmail.com>
**
** Ce programme est libre, vous pouvez le redistribuer et/ou le modifier 
** selon les termes de la Licence Publique Gnrale GNU publie par la Free Software Foundation 
**
** Ce programme est distribu car potentiellement utile, mais SANS AUCUNE GARANTIE, ni explicite ni implicite, 
** y compris les garanties de commercialisation ou d'adaptation dans un but spcifique.
** Reportez-vous  la Licence Publique Gnrale GNU pour plus de dtails.
**
** Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en mme temps que ce programme 
**
** game.c
**
** Gestion de tout ce qui concerne le jeu.
**
**
*/

# include "game.h"

/*
** Dbut du jeu
*/
void StartGame()
{
	SDL_Event event;
	unsigned int gameover = 0;

	struct s_bonus bonus;
	struct s_walls walls;

	InitGame();
	InitBalls();
	InitBonus(&bonus);
	InitWalls(&walls);
	InitMessages(gameui.messages);

	AddRandBonus(&bonus);	/* Ajout d'un bonus */

	while (!gameover) {
		/* Attente d'un evenement pour faire quoique ce soit */
		while (SDL_PollEvent(&event)) {
			EventGame(&event, &gameover);	/* gestion des evenements du jeu (pause) */
			EventBalls(&event);
		}

		HandleGameBg();	/* Fond */
		HandleBonus(&bonus, &walls);	/* Bonus */
		HandleWalls(&walls, &bonus);	/* Murs */
		HandleBalls(&bonus, &walls);	/* Balles */
		HandleMessages(gameui.messages);
		HandleGameFg();	/* Premier plan */

		IsGameOver(&gameover);

		SDL_Flip(SDL_GetVideoSurface());
		limitFps();	/* Limite le nombre de fps pour conomiser un peu de cpu */
	}

	ShowScore(&walls);

	FreeBalls();
	FreeBonus(&bonus);
	FreeWalls(&walls);
	FreeMessages(gameui.messages);
	FreeGame();
}

/*
** Initialisation du Jeu
*/
void InitGame()
{
	unsigned int i;
	char tmp[256];

	/* Fond */
	sprintf(tmp, DATA_PATH GAME_IMG_BGTILE);
	gameui.bgtile = LoadSDLImage(tmp, false);

	/* Balles */
	gameui.balls = malloc(options.playernb * sizeof(SDL_Surface));
	for (i = 0; i < options.playernb; i++) {
		sprintf(tmp, DATA_PATH GAME_IMG_BALL, (i + 1));
		gameui.balls[i] = LoadSDLImage(tmp, true);
	}

	/* Murs */
	gameui.walls = LoadSDLImage(DATA_PATH GAME_IMG_WALLS, true);

	/* Bonus */
	gameui.bonus = LoadSDLImage(DATA_PATH GAME_IMG_BONUS, true);

	/* chargement des polices utilises dans le jeu */
	gameui.font = LoadSDLFont(DATA_PATH GAME_FONT, GAME_FONT_SIZE);
	gameui.fontBig =
	    LoadSDLFont(DATA_PATH GAME_FONT, GAME_FONT_SIZE_BIG);

	/* Chargement des sons */
	gameui.wavgameover = LoadSDLWav(DATA_PATH GAME_SOUND_GAMEOVER);
	gameui.wavbonus = LoadSDLWav(DATA_PATH GAME_SOUND_BONUS);

	/* Messages */
	gameui.messages = malloc(sizeof(struct s_messages));
	gameui.messages->first = NULL;
	gameui.messages->last = NULL;
}

/*
** Libration de la mmoire
*/
void FreeGame()
{
	unsigned int i;

	SDL_FreeSurface(gameui.background);
	gameui.background = NULL;

	for (i = 0; i < options.playernb; i++) {
		SDL_FreeSurface(gameui.balls[i]);
		gameui.balls[i] = NULL;
	}

	free(gameui.balls);

	SDL_FreeSurface(gameui.walls);
	gameui.walls = NULL;

	SDL_FreeSurface(gameui.bonus);
	gameui.bonus = NULL;

	TTF_CloseFont(gameui.font);	/* Police normale */
	gameui.font = NULL;

	TTF_CloseFont(gameui.fontBig);	/* Police plus grosse */
	gameui.fontBig = NULL;

	Mix_FreeChunk(gameui.wavbonus);
	gameui.wavbonus = NULL;
	Mix_FreeChunk(gameui.wavgameover);
	gameui.wavgameover = NULL;
}

/*
** Affichage du fond
*/
void HandleGameBg()
{
	SDL_Rect src, dst;
	unsigned int i, j;

	src.x = src.y = 0;
	src.w = GAME_BG_TILE_WIDTH;
	src.h = GAME_BG_TILE_HEIGHT;
	dst.w = src.w;
	dst.h = src.h;

	/* On met le fond */
	for (i = 0; i < SCREEN_WIDTH; i += GAME_BG_TILE_WIDTH) {
		for (j = 0; j < SCREEN_WIDTH; j += GAME_BG_TILE_HEIGHT) {
			dst.x = i;
			dst.y = j;
			SDL_BlitSurface(gameui.bgtile, &src,
					SDL_GetVideoSurface(), &dst);
		}
	}

}

/*
** Affichage du premier plan
*/
void HandleGameFg()
{
	unsigned int i;
	SDL_Rect dst;

	SDL_Color color;

	/* FPS (inutile) */
	char tmp[256];

	sprintf(tmp, "FPS %d", nb_fps);
	dst.x = SCREEN_WIDTH - 60;
	dst.y = 1;
	PrintText(SDL_GetVideoSurface(), tmp, &dst, gameui.font,
		  colors.sdl_grey);

	/* Pour tout les joueurs */
	for (i = 0; i < options.playernb; i++) {
		dst.x = 5;
		dst.y = 5 + i * 25;
		/* Couleur suivant l'tat du joueur */
		if (balls[i].lifes <= 0)
			color = colors.sdl_red;
		else if (balls[i].invincibletime > 0)
			color = colors.sdl_blue;
		else
			color = colors.sdl_white;


		if (balls[i].type == BALL_PLAYER)
			sprintf(tmp, "Player %d", i + 1);
		else if (balls[i].type == BALL_IA)
			sprintf(tmp, "AI %d", i + 1);

		PrintText(SDL_GetVideoSurface(), tmp, &dst, gameui.fontBig,
			  color);

		dst.x += dst.w;
		sprintf(tmp, "Lifes: %d - Speed: %d", balls[i].lifes,
			(int) balls[i].speed);
		PrintText(SDL_GetVideoSurface(), tmp, &dst, gameui.font,
			  colors.sdl_grey);

		dst.y += dst.h;
		sprintf(tmp, "Score: %d", balls[i].score);
		PrintText(SDL_GetVideoSurface(), tmp, &dst, gameui.font,
			  colors.sdl_grey);

		if (balls[i].invincibletime > 0 && balls[i].lifes > 0) {
			dst.x += 5 + dst.w;
			dst.w = balls[i].invincibletotaltime / 100;
			DrawGraph(SDL_GetVideoSurface(), dst,
				  balls[i].invincibletime,
				  balls[i].invincibletotaltime);
		}

	}
	/* FIXME Rajouter un rectangle transparent derriere */
}

/*
** Evenements du jeu 
*/
void EventGame(SDL_Event * event, unsigned int *gameover)
{
	switch (event->type) {
	case SDL_KEYDOWN:
		if (event->key.state == SDL_PRESSED) {
			if (event->key.keysym.sym == SDLK_ESCAPE)
				*gameover = 1;
			else if (event->key.keysym.sym == SDLK_p) {
				PauseGame(gameover);
			}
			/* FIXME: Rajouter des trucs pour tricher */
		}
		break;
	case SDL_QUIT:
		*gameover = 1;
		break;
	}
}

/*
** Met le jeu en pause
*/
void PauseGame(unsigned int *gameover)
{
	SDL_Event event;
	SDL_Rect dst;
	unsigned int pause = 1;

	/* Tant que c'est en pause et qu'on quitte pas */
	while (pause && !*gameover) {
		/* Attente d'un evenement pour faire quoique ce soit */
		while (SDL_PollEvent(&event)) {
			switch (event.type) {
			case SDL_KEYDOWN:
				if (event.key.state == SDL_PRESSED) {
					if (event.key.keysym.sym == SDLK_p) {
						pause = 0;
					}
				}
				break;
			case SDL_QUIT:
				*gameover = 1;
				break;
			}
		}

		/* Fond */
		HandleGameBg();


		dst.x = SCREEN_WIDTH / 2;
		dst.y = SCREEN_HEIGHT / 2;
		PrintText(SDL_GetVideoSurface(), "PAUSE", &dst,
			  gameui.fontBig, colors.sdl_red);

		/* Premier plan */
		HandleGameFg();

		SDL_Flip(SDL_GetVideoSurface());
		limitFps();	/* Limite le nombre de fps pour conomiser un peu de cpu */
	}

}

/*
** Verifie si la partie est finie
*/
void IsGameOver(unsigned int *gameover)
{
	unsigned int i;

	for (i = 0; i < options.playernb; ++i)
		if (balls[i].lifes)
			return;

	/* La partie est finie, plus personne n'a de vie */
	PlaySDLWav(gameui.wavgameover);
	*gameover = 1;
}

/*
** Affichage des scores 
*/
void ShowScore(struct s_walls *walls)
{
	SDL_Event event;
	SDL_Rect dst, dst2, src;
	char tmp[256];
	unsigned int done = 0;
	unsigned int i, j;
	int win = -1;
	unsigned int max = 0;

	/* Tant que c'est pas finit */
	while (!done) {
		/* Attente d'un evenement pour faire quoique ce soit */
		while (SDL_PollEvent(&event)) {
			switch (event.type) {
			case SDL_KEYDOWN:
				if (event.key.state == SDL_PRESSED) {
					if (event.key.keysym.sym ==
					    SDLK_ESCAPE)
						done = 1;
				}
				break;
			case SDL_QUIT:
				done = 1;
				break;
			}
		}

		/* Affiche le fond */
		HandleGameBg();


		dst.x = SCREEN_WIDTH - 300;
		dst.y = 50;

		/* Pour chaque joueur */
		for (i = 0; i < options.playernb; ++i) {
			/* Pour les images */
			src.x = src.y = 0;
			dst2 = dst;

			if (balls[i].type == BALL_PLAYER)
				sprintf(tmp, "Player %d: %d points", i + 1,
					balls[i].score);
			else if (balls[i].type == BALL_IA)
				sprintf(tmp, "AI %d: %d points", i + 1,
					balls[i].score);

			PrintText(SDL_GetVideoSurface(), tmp, &dst,
				  gameui.fontBig, colors.sdl_white);

			/* balle */
			src.w = BALL_WIDTH;
			src.h = BALL_HEIGHT;
			dst2.h = src.h;
			dst2.w = src.w;
			dst2.x -= dst2.w;
			SDL_BlitSurface(gameui.balls[i], &src,
					SDL_GetVideoSurface(), &dst2);

			if (max < balls[i].score) {	/* Calcul du gagnant */
				max = balls[i].score;
				win = i;
			}

			dst.y += 5;
			/* Pour chaque type de bonus */
			for (j = 0; j < BONUS_NB; ++j) {
				if (balls[i].bonuscount[j] > 0) {
					dst.y += BONUS_HEIGHT + 2;
					sprintf(tmp, "%s: %d",
						bonusconf[j].name,
						balls[i].bonuscount[j]);
					PrintText(SDL_GetVideoSurface(),
						  tmp, &dst, gameui.font,
						  colors.sdl_white);

					/* dessin du bonus */
					src.h = BONUS_HEIGHT;
					src.w = BONUS_WIDTH;
					src.x = j * BONUS_WIDTH;
					dst2.h = src.h;
					dst2.w = src.w;
					dst2.x = dst.x - dst2.w - 5;
					dst2.y = dst.y;
					SDL_BlitSurface(gameui.bonus, &src,
							SDL_GetVideoSurface
							(), &dst2);
				}
			}
			dst.y += 35;
		}

		sprintf(tmp, "Walls: %d", walls->wallsnb);
		PrintText(SDL_GetVideoSurface(), tmp, &dst, gameui.font,
			  colors.sdl_white);
		dst.y += 35;

		/* Si y'a plus d'un joueur, on affiche le gagnant */
		if (options.playernb > 1 && win >= 0) {
			dst.x = 200;
			dst.y = 200;

			if (balls[win].type == BALL_PLAYER)
				sprintf(tmp, "Winner: Player %d", win + 1);
			else if (balls[win].type == BALL_IA)
				sprintf(tmp, "Winner: AI %d", win + 1);

			PrintText(SDL_GetVideoSurface(), tmp, &dst,
				  gameui.fontBig, colors.sdl_white);

			/* balle */
			dst2 = dst;
			src.w = BALL_WIDTH;
			src.h = BALL_HEIGHT;
			dst2.h = src.h;
			dst2.w = src.w;
			dst2.x += dst.w;
			SDL_BlitSurface(gameui.balls[win], &src,
					SDL_GetVideoSurface(), &dst2);
		}

		dst.y += 20;
		PrintText(SDL_GetVideoSurface(),
			  "ESC to come back to the menu", &dst,
			  gameui.font, colors.sdl_white);

		/* Premier Plan */
		HandleGameFg();

		SDL_Flip(SDL_GetVideoSurface());
		limitFps();	/* Limite le nombre de fps pour conomiser un peu de cpu */
	}
}
