/*
** Copyright (C) 2005 CHARY Corentin <iksaif@gmail.com>
**
** Ce programme est libre, vous pouvez le redistribuer et/ou le modifier 
** selon les termes de la Licence Publique Gnrale GNU publie par la Free Software Foundation 
**
** Ce programme est distribu car potentiellement utile, mais SANS AUCUNE GARANTIE, ni explicite ni implicite, 
** y compris les garanties de commercialisation ou d'adaptation dans un but spcifique.
** Reportez-vous  la Licence Publique Gnrale GNU pour plus de dtails.
**
** Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en mme temps que ce programme 
**
** menu.c
**
** Gestion de l'interface
**
**
*/

# include "menu.h"

/*
** Fonction qui lance le menu->
*/

void StartMenu()
{
	SDL_Event event;
	struct s_menu menu;
	unsigned int isdone = 0;

	InitMenu(&menu);


	while (!isdone) {
		/* Attente d'un evenement pour faire quoique ce soit */
		while (SDL_PollEvent(&event)) {
			EventMenu(&event, &menu, &isdone);	/* gestion des evenements autres que quitter */
		}

		HandleMenu(&menu);	/* gestion de l'affichage du menu */

		SDL_Flip(SDL_GetVideoSurface());
		limitFps();	/* Limite le nombre de fps pour conomiser un peu de cpu */
	}

	FreeMenu(&menu);
}

/*
** Fonction permetant d'initialiser les variables du menu
*/
void InitMenu(struct s_menu *menu)
{
	char tmp[256];

	/* tout les boutons sont en mode "non-actif" */
	for (uint i = 0; i < MENU_BUTTON_NB; i++)
		menu->buttons[i].state = 0;

	/*Chargement de l'image de fond */
	sprintf(tmp, DATA_PATH MENU_IMG_BACKGROUND, SCREEN_WIDTH,
		SCREEN_HEIGHT);
	menu->bg = LoadSDLImage(tmp, false);

	/* Chargement des elements graphiques */
	menu->surf = LoadSDLImage(DATA_PATH MENU_IMG_ELEM, true);

	/* chargement de la police utilise dans le jeu */
	menu->font = LoadSDLFont(DATA_PATH GAME_FONT, MENU_FONT_SIZE);

	/* chargement de la police utilise dans le jeu */
	menu->fontBig =
	    LoadSDLFont(DATA_PATH GAME_FONT, MENU_FONT_SIZE_BIG);

	/* Catgories, options, etc .... */
	InitCat(&menu->cat, "Ikswall", 3, NULL);
	AddItemFct(&menu->cat, "Play", StartGame);
	AddItemLnk(&menu->cat, "Options", NULL);
	AddItemTxt(&menu->cat, "______");
	AddItemFct(&menu->cat, "Quit", QuitMenu);

	InitCat(&menu->cat[1].lnk, menu->cat[1].name, 2, &menu->cat);
	AddItemOpt(menu->cat[1].lnk, NULL, options.playernb);
	AddItemOpt(menu->cat[1].lnk, NULL, options.ianb);
	AddItemOpt(menu->cat[1].lnk, NULL, options.speed);
	AddItemOpt(menu->cat[1].lnk, NULL, options.wallsize);
	AddItemOpt(menu->cat[1].lnk, NULL, options.mode);
	AddItemOpt(menu->cat[1].lnk, NULL, options.vresol);
	AddItemOpt(menu->cat[1].lnk, NULL, options.vmode);
	AddItemFct(menu->cat[1].lnk, "Save Options", SaveOptions);
	AddItemLnk(menu->cat[1].lnk, "Back", &menu->cat);
}

/*
** Fonction permetant de liberer l'espace mmoire allou au menu
*/
void FreeMenu(struct s_menu *menu)
{
	SDL_FreeSurface(menu->surf);	/* Elements graphiques */
	menu->surf = NULL;
	SDL_FreeSurface(menu->bg);	/* Fond */
	menu->bg = NULL;
	TTF_CloseFont(menu->font);	/* Police normale */
	menu->font = NULL;
	TTF_CloseFont(menu->fontBig);	/* Grande Police */
	menu->fontBig = NULL;
	FreeCat(menu->cat);
	menu->cat = NULL;
}

/*
** Libre une catgorie (et la parcoure pour librer le reste)
*/
void FreeCat(struct s_menu_cat *cat) {

}

/*
** Fonction qui gre l'affichage du menu
*/
void HandleMenu(struct s_menu *menu)
{
	SDL_Rect src, dst;
	char tmp[256];
	unsigned int i;

	/* Image de fond */
	src.x = src.y = 0;
	src.w = menu->bg->w;
	src.h = menu->bg->h;
	dst.x = dst.y = 0;
	dst.w = src.w;
	dst.h = src.h;
	SDL_BlitSurface(menu->bg, &src, SDL_GetVideoSurface(), &dst);

	/* Boutons */

	/*
	 ** A droite
	 */
	/* Jouer */
	dst.x = screen->w - 200;
	dst.y = 50;
	DrawButton(menu, 0, "Start", &dst);

	/* Quitter */
	dst.y = 550;
	DrawButton(menu, 1, "Quit", &dst);

	/*
	 ** Au Centre
	 */
	dst.x = 20;
	dst.y = 25;
	PrintText(SDL_GetVideoSurface(), "Options:", &dst, menu->fontBig,
		  colors.sdl_grey);

	dst.x = 20;
	dst.y = 75;
	sprintf(tmp, "%d", options.playernb);
	DrawOption(menu, 2, "Players: ", tmp, &dst);

	dst.y = 125;
	sprintf(tmp, "%d", options.ianb);
	DrawOption(menu, 4, "Computers: ", tmp, &dst);

	dst.y = 175;
	sprintf(tmp, "%d", (unsigned int) options.speed);
	DrawOption(menu, 6, "Speed: ", tmp, &dst);

	dst.y = 225;
	sprintf(tmp, "%d", options.wallsize);
	DrawOption(menu, 8, "Wall Size: ", tmp, &dst);

	dst.y = 275;
	sprintf(tmp, "%s", modes[options.mode].name);
	DrawOption(menu, 10, "Mode: ", tmp, &dst);

	/*
	 ** Vido
	 */
	dst.x = 20;
	dst.y = 325;
	PrintText(SDL_GetVideoSurface(), "Video:", &dst, menu->fontBig,
		  colors.sdl_grey);

	dst.y = 375;
	sprintf(tmp, "%dx%d", SCREEN_WIDTH, SCREEN_HEIGHT);
	DrawOption(menu, 12, "Resolution: ", tmp, &dst);

	dst.y = 425;
	sprintf(tmp, "%s", vmodes[options.vmode]);
	DrawOption(menu, 14, "Mode: ", tmp, &dst);

	dst.y = 475;
	DrawButton(menu, 16, "Apply", &dst);

	/*
	 ** Divers
	 */
	/* version (inutile) */
	sprintf(tmp, "v %s", VERSION);
	dst.x = screen->w - 30;
	dst.y = screen->h - 11;
	PrintText(SDL_GetVideoSurface(), tmp, &dst, menu->font,
		  colors.sdl_grey);


	/* FPS (inutile) */
	sprintf(tmp, "FPS %d", nb_fps);
	dst.x = screen->w - 60;
	dst.y = 1;
	PrintText(SDL_GetVideoSurface(), tmp, &dst, menu->font,
		  colors.sdl_grey);

	/* Touches */
	for (i = 0; i < options.playernb; i++) {
		dst.x = 300;
		dst.y = 75 * (i + 1);

		if (options.playernb - (i + 1) < options.ianb)
			sprintf(tmp, "AI %d", i + 1);
		else
			sprintf(tmp, "Player %d", i + 1);

		PrintText(SDL_GetVideoSurface(), tmp, &dst, menu->font,
			  colors.sdl_grey);

		dst.x += 100;
		DrawKeyMod(menu, options.keys[i][OPTK_UP], &dst);
		dst.y += BUTTON3_HEIGHT;
		DrawKeyMod(menu, options.keys[i][OPTK_DOWN], &dst);
		dst.x -= BUTTON3_WIDTH;
		DrawKeyMod(menu, options.keys[i][OPTK_LEFT], &dst);
		dst.x += BUTTON3_WIDTH * 2;
		DrawKeyMod(menu, options.keys[i][OPTK_RIGHT], &dst);
	}
}


/*
** Dessine de quoi rgler une touche
*/
void DrawKeyMod(struct s_menu *menu, unsigned int keysym, SDL_Rect * orig)
{
	SDL_Rect src, dst;

	dst = *orig;

	src.x = BUTTON1_WIDTH + BUTTON2_WIDTH;
	src.y = 0;
	src.w = dst.w = BUTTON3_WIDTH;
	src.h = dst.h = BUTTON3_HEIGHT;

	dst.y -= (BUTTON3_HEIGHT - MENU_FONT_SIZE) / 2;
	/* Dessin du "bouton" */
	SDL_BlitSurface(menu->surf, &src, SDL_GetVideoSurface(), &dst);

	dst.x += 7;
	dst.y += 7;
	/* Texte */
	PrintText(SDL_GetVideoSurface(), SDL_GetKeyName(keysym), &dst,
		  menu->font, colors.sdl_grey);
}

/*
** Dessine un bouton
*/
void
DrawButton(struct s_menu *menu, unsigned int id, char *title,
	   SDL_Rect * orig)
{
	SDL_Rect src, dst;

	dst = *orig;

	src.w = dst.w = BUTTON1_WIDTH;
	src.h = dst.h = BUTTON1_HEIGHT;
	src.x = 0;
	src.y = menu->buttons[id].state * BUTTON1_HEIGHT;
	/* Bouton */
	SDL_BlitSurface(menu->surf, &src, SDL_GetVideoSurface(), &dst);

	/* On stoque la position */
	menu->buttons[id].pos = dst;

	dst.x += 10 + menu->buttons[id].state * 1;
	dst.y += 7 + menu->buttons[id].state * 1;
	/* Texte */
	PrintText(SDL_GetVideoSurface(), title, &dst, menu->font,
		  colors.sdl_grey);
}

/*
** Dessine une option avec + et -
*/
void
DrawOption(struct s_menu *menu, unsigned int id, char *title,
	   char *optvalue, SDL_Rect * orig)
{
	SDL_Rect src, dst;
	unsigned int titlew;
	unsigned int optw;

	dst = *orig;

	PrintText(SDL_GetVideoSurface(), title, &dst, menu->font,
		  colors.sdl_grey);

	titlew = dst.w;

	src.x = BUTTON1_WIDTH;
	src.w = dst.w = BUTTON2_WIDTH;
	src.h = dst.h = BUTTON2_HEIGHT;

	src.y = menu->buttons[id].state * BUTTON2_HEIGHT;
	dst.x += titlew;
	dst.y -= (BUTTON2_HEIGHT - MENU_FONT_SIZE) / 2;
	SDL_BlitSurface(menu->surf, &src, SDL_GetVideoSurface(), &dst);
	menu->buttons[id].pos = dst;

	dst.x += BUTTON2_WIDTH / 2 + menu->buttons[id].state * 1;
	dst.y += BUTTON2_HEIGHT / 3 + menu->buttons[id].state * 1;
	PrintText(SDL_GetVideoSurface(), "-", &dst, menu->font,
		  colors.sdl_grey);


	dst.x += dst.w + BUTTON2_WIDTH + 10;
	PrintText(SDL_GetVideoSurface(), optvalue, &dst, menu->font,
		  colors.sdl_grey);
	optw = dst.w;


	src.y = menu->buttons[id + 1].state * BUTTON2_HEIGHT;
	dst.x += optw + BUTTON2_WIDTH;
	dst.y = orig->y - (BUTTON2_HEIGHT - MENU_FONT_SIZE) / 2;
	SDL_BlitSurface(menu->surf, &src, SDL_GetVideoSurface(), &dst);
	menu->buttons[id + 1].pos = dst;

	dst.x += BUTTON2_WIDTH / 2 + menu->buttons[id + 1].state * 1 - 3;
	dst.y += BUTTON2_HEIGHT / 3 + menu->buttons[id + 1].state * 1;
	PrintText(SDL_GetVideoSurface(), "+", &dst, menu->font,
		  colors.sdl_grey);

}

/*
** Gre les evenements sur le menu (click et clavier)
*/
void
EventMenu(SDL_Event * event, struct s_menu *menu, unsigned int *isdone)
{
	if (!event)
		return;

	static unsigned int mouseButtonDown;
	unsigned int i;
	unsigned int buttonLaunched = 0;
	int activeButton = -1;

	static unsigned int good_vmode = OPT_VMODE;
	static unsigned int good_vresol = OPT_VRESOL;

	char tmp[256];
	unsigned int flags;

	/* Tout les boutons sont mis a 0, a evite de tester plusieurs trucs */
	for (i = 0; i < MENU_BUTTON_NB; i++) {
		if (menu->buttons[i].state)
			activeButton = i;

		menu->buttons[i].state = 0;
	}

	switch (event->type) {
	case SDL_KEYDOWN:	/* Touche appuye */
		if (event->key.state == SDL_PRESSED) {
			if (event->key.keysym.sym == SDLK_ESCAPE)
				*isdone = 1;	/* Echap = quitter */
			else if (event->key.keysym.sym == SDLK_UP) {	/* HAUT et  BAS permettent de selectionner les boutons */
				activeButton--;
				if (activeButton == -2)
					activeButton = 0;
				if (activeButton < 0)
					activeButton = MENU_BUTTON_NB - 1;
			} else if (event->key.keysym.sym == SDLK_DOWN) {
				activeButton++;
				if (activeButton >= MENU_BUTTON_NB)
					activeButton = 0;
			} else if (event->key.keysym.sym == SDLK_RETURN) {	/* RETOUR lance l'action associe  un bouton */
				buttonLaunched = 1;
			}
		}
		break;
	case SDL_QUIT:
		*isdone = 1;
		break;
	case SDL_MOUSEBUTTONDOWN:	/* si on click */
		if (event->button.button == SDL_BUTTON_LEFT) {
			mouseButtonDown = 1;

			/* Parcour des boutons */
			for (i = 0; i < MENU_BUTTON_NB; i++) {
				if (event->button.x >=
				    menu->buttons[i].pos.x
				    && event->button.x <=
				    menu->buttons[i].pos.x +
				    menu->buttons[i].pos.w
				    && event->button.y >=
				    menu->buttons[i].pos.y
				    && event->button.y <=
				    menu->buttons[i].pos.y +
				    menu->buttons[i].pos.h) {
					activeButton = i;
					break;
				}

			}
		}
		break;

	case SDL_MOUSEBUTTONUP:	/* si on relache le bouton */
		if (event->button.button == SDL_BUTTON_LEFT) {
			mouseButtonDown = 0;
			if (activeButton >= 0)	/* Si un bouton tait actif, on lance l'action associe */
				buttonLaunched = 1;
		}
		break;

	case SDL_MOUSEMOTION:
		/* Modifie les boutons en fonctions de l'tat et de la position de la souris */
		if (mouseButtonDown) {

			for (i = 0; i < MENU_BUTTON_NB; i++) {
				if (event->motion.x >=
				    menu->buttons[i].pos.x
				    && event->motion.x <=
				    menu->buttons[i].pos.x +
				    menu->buttons[i].pos.w
				    && event->motion.y >=
				    menu->buttons[i].pos.y
				    && event->motion.y <=
				    menu->buttons[i].pos.y +
				    menu->buttons[i].pos.h) {
					activeButton = i;
					break;
				}

			}
		}
		break;
	}

	if (activeButton >= 0)	/* Si on bouton est actif, on l'active dans la structure menu */
		menu->buttons[activeButton].state = 1;

	if (buttonLaunched) {	/* Lancement de l'action associe  un bouton */
		switch (activeButton) {
		case 0:	/* Jouer */
			if (screen->w != SCREEN_WIDTH || screen->h != SCREEN_HEIGHT) {	/* Au cas ou on aurait chang la rsolution sans appuyer sur "Appliquer" */
				if (options.vmode == 1)
					flags =
					    SDL_DOUBLEBUF | SDL_FULLSCREEN;
				else
					flags = SDL_DOUBLEBUF;

				if (SetVideoMode
				    (SCREEN_WIDTH, SCREEN_HEIGHT, 32,
				     flags)) {
					good_vmode = options.vmode;
					good_vresol = options.vresol;
				} else {
					options.vmode = good_vmode;
					options.vresol = good_vresol;
				}

				sprintf(tmp, DATA_PATH MENU_IMG_BACKGROUND,
					SCREEN_WIDTH, SCREEN_HEIGHT);
				menu->bg = LoadSDLImage(tmp, false);
			}
			StartGame();
			break;
		case 1:	/* Quitter */
			*isdone = 1;
			break;
		case 2:	/* Joueur "-" */
			if (options.playernb > OPTMIN_PLAYER_NB) {
				options.playernb--;
				if (options.playernb < options.ianb)
					options.ianb = options.playernb;
			}
			break;
		case 3:	/* Joueur "+" */
			if (options.playernb < OPTMAX_PLAYER_NB)
				options.playernb++;
			break;
		case 4:	/* IA "-" */
			if (options.ianb > OPTMIN_IA_NB)
				options.ianb--;
			break;
		case 5:	/* IA "+" */
			if (options.ianb < options.playernb
			    && options.ianb < OPTMAX_IA_NB)
				options.ianb++;
			break;
		case 6:	/* Vitesse "-" */
			if (options.speed > OPTMIN_SPEED)
				options.speed--;
			break;
		case 7:	/* Vitesse "+" */
			if (options.speed < OPTMAX_SPEED)
				options.speed++;
			break;
		case 8:	/* Taille "-" */
			if (options.wallsize > OPTMIN_WALLSIZE)
				options.wallsize--;
			break;
		case 9:	/* Taille "+" */
			if (options.wallsize < OPTMAX_WALLSIZE)
				options.wallsize++;
			break;
		case 10:	/* Mode "-" */
			if (options.mode > OPTMIN_MODE)
				options.mode--;
			break;
		case 11:	/* Mode "+" */
			if (options.mode < OPTMAX_MODE)
				options.mode++;
			break;
		case 12:	/* Resol "-" */
			if (options.vresol > OPTMIN_VRESOL)
				options.vresol--;
			break;
		case 13:	/* Resol "+" */
			if (options.vresol < OPTMAX_VRESOL)
				options.vresol++;
			break;
		case 14:	/* Mode "+" */
			if (options.vmode > OPTMIN_VMODE)
				options.vmode--;
			break;
		case 15:	/* Mode "+" */
			if (options.vmode < OPTMAX_VMODE)
				options.vmode++;
			break;
		case 16:	/* Appliquer changements videos */
			if (options.vmode == 1)
				flags = SDL_DOUBLEBUF | SDL_FULLSCREEN;
			else
				flags = SDL_DOUBLEBUF;

			if (SetVideoMode
			    (SCREEN_WIDTH, SCREEN_HEIGHT, 32, flags)) {
				good_vmode = options.vmode;
				good_vresol = options.vresol;
			} else {
				options.vmode = good_vmode;
				options.vresol = good_vresol;
			}

			sprintf(tmp, DATA_PATH MENU_IMG_BACKGROUND,
				SCREEN_WIDTH, SCREEN_HEIGHT);
			menu->bg = LoadSDLImage(tmp, false);
			break;
		}

		menu->buttons[activeButton].state = 0;
		activeButton = -1;
	}
}

void DrawCat(struct s_menu_cat *cat, TTF_Font *font, SDL_Color catcolor, SDL_Color actcolor, SDL_Color inactcolor) {

}

void DrawItem(struct s_menu_item *item, TTF_Font *font, SDL_Color *color) {

}

void InitCat(struct s_menu_cat **cat, char * name, uint itemsnb, struct s_menu_cat *parent) {
	*cat = malloc(sizeof(struct s_menu_cat));
	
	(*cat)->name = malloc(strlen(name)+1);
	strcpy(cat.name,name);
	
	(*cat)->itemsnb = 0;
	(*cat)->itemsmax = itemsnb;
	(*cat)->items = malloc(sizeof(s_menu_item)*itemsnb);
	
	(*cat)->parent = parent;
}

void AddItemTxt(struct s_menu_cat *cat, char * text) {	
	
	if(cat->itemsnb >= cat->itemsmax) {
		fprintf(stderr,"Too much items in cat: %s\n",cat->name);
		return;
	}
	
	cat->items[cat->itemsnb].text = malloc(strlen(text)+1);
	strcpy(cat->items[cat->itemsnb].text,text);
	
	cat->itemsnb++;
}

void AddItemOpt(struct s_menu_cat *cat, char * text, struct opt opt) {
	if(cat->itemsnb >= cat->itemsmax) {
		fprintf(stderr,"Too much items in cat: %s\n",cat->name);
		return;
	}
	
	if(text) {
		cat->items[cat->itemsnb].text = malloc(strlen(text)+1);
		strcpy(cat->items[cat->itemsnb].text,text);
	} else {
		cat->items[cat->itemsnb].text = malloc(strlen(opt.text)+1);
		strcpy(cat->items[cat->itemsnb].text,opt.text);
	}
	
	cat->itemsnb++;
}

void AddItemFct(struct s_menu_cat *cat, char * text, void *fct) {
	if(cat->itemsnb >= cat->itemsmax) {
		fprintf(stderr,"Too much items in cat: %s\n",cat->name);
		return;
	}
	
	cat->items[cat->itemsnb].text = malloc(strlen(text)+1);
	strcpy(cat->items[cat->itemsnb].text,text);
	
	cat->fct = fct;
	
	cat->itemsnb++;
}

void AddItemLnk(struct s_menu_cat *cat, char * text, struct s_menu_cat *lnk) {
	if(cat->itemsnb >= cat->itemsmax) {
		fprintf(stderr,"Too much items in cat: %s\n",cat->name);
		return;
	}
	
	cat->items[cat->itemsnb].text = malloc(strlen(text)+1);
	strcpy(cat->items[cat->itemsnb].text,text);
	
	cat->lnk = lnk;
	
	cat->itemsnb++;
}
