/*
**
** Ce programme est libre, vous pouvez le redistribuer et/ou le modifier
** selon les termes de la Licence Publique Gnrale GNU publie par la Free Software Foundation
**
** Ce programme est distribu car potentiellement utile, mais SANS AUCUNE GARANTIE, ni explicite ni implicite,
** y compris les garanties de commercialisation ou d'adaptation dans un but spcifique.
** Reportez-vous  la Licence Publique Gnrale GNU pour plus de dtails.
**
** Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en mme temps que ce programme
**
** messages.h
**
** Permet d'afficher des messages un peu partout sur le jeu
** Avec des effets (fondu, clignotement, etc ...)
**
**
*/

# include "messages.h"

void InitMessages(struct s_messages *messages)
{
        messages->first = NULL;
        messages->last = NULL;
}


void FreeMessages(struct s_messages *messages)
{
        struct s_message *tmp;
        struct s_message *pmessage;

        /* Virer les bonus, parcour de la liste chane */
        pmessage = messages->first;
        while (pmessage) {
                tmp = pmessage;
                pmessage = pmessage->next;
                free(tmp);
        }
        messages->first = NULL;
        messages->last = NULL;
}

void HandleMessages(struct s_messages *messages)
{
        struct s_message *pmessage,*next;
	SDL_Surface *text_surf;
	SDL_Rect dst;
	SDL_Color color={0,0,0};

        /* Virer les bonus, parcour de la liste chane */
        pmessage = messages->first;
        while (pmessage) {

		next = pmessage->next;

		if(pmessage->time < SDL_GetTicks()) {
			DelMessage(messages, pmessage);
		} else {
			pmessage->x = (pmessage->x+pmessage->dx) % SCREEN_WIDTH;
			pmessage->y = (pmessage->y+pmessage->dy) % SCREEN_HEIGHT;

			dst.x = pmessage->x;
                        dst.y = pmessage->y;

			if(pmessage->flags&MESSAGE_FADING) {
				/* FIXME */
				color = pmessage->color;
			} else if(pmessage->flags&MESSAGE_BLITTING) {
				pmessage->color.r = (pmessage->color.r + myrand(10,20)) % 255;
				pmessage->color.g = (pmessage->color.g + myrand(10,20)) % 255;
				pmessage->color.b = (pmessage->color.b + myrand(10,20)) % 255;	
				color = pmessage->color;
			} else {
				color = pmessage->color;
			}
			
			text_surf = TTF_RenderText_Blended(pmessage->font, pmessage->text, color);
			SDL_BlitSurface(text_surf, NULL, SDL_GetVideoSurface(), &dst);
			SDL_FreeSurface(text_surf);

			
		}

		pmessage = next;
        }
}


void AddMessage(struct s_messages *messages, char text[256], SDL_Color color,
		TTF_Font * font, unsigned int x, unsigned int y,
		int dx, int dy, unsigned int flags,
		unsigned int time)
{
	struct s_message *new;
	new = malloc(sizeof(struct s_message));

	new->prev = NULL;
	if (messages->first == NULL)	/* Il n'y a pas de premier (chaine vide) */
		new->next = NULL;
	else			/* Sinon */
		new->next = messages->first;


	if (messages->last == NULL)	/* Il n'y pas de dernier (chaine vide) */
		messages->last = new;
	else			/* Sinon */
		new->next->prev = new;

	messages->first = new;	/* Le nouveau bonus mis au dbut */
	
	/* Reglages */
	strcpy(new->text,text);
	
	if(flags&MESSAGE_FADING) {
		new->color.r = new->color.g = new->color.b = 0;
		new->color.unused = (int) ceil((double) time / 255);
	} else {
		new->color = color;
	}

	new->font = font;
	
	new->x = x;
	new->y = y;
	
	new->dy = dy;
	new->dx = dx;

	new->flags = flags;

	new->time = SDL_GetTicks() + time;
      
}

void DelMessage(struct s_messages *messages, struct s_message *message)
{
	if (message->prev == NULL)	/* Si c'tait le premier */
		messages->first = message->next;
	else			/* Sinon */
		message->prev->next = message->next;

	if (message->next == NULL)	/* Si c'tait le dernier */
		messages->last = message->prev;
	else			/* Sinon */
		message->next->prev = message->prev;

	free(message);
}
