/*
** Copyright (C) 2005 CHARY Corentin <iksaif@gmail.com>
**
** Ce programme est libre, vous pouvez le redistribuer et/ou le modifier 
** selon les termes de la Licence Publique Gnrale GNU publie par la Free Software Foundation 
**
** Ce programme est distribu car potentiellement utile, mais SANS AUCUNE GARANTIE, ni explicite ni implicite, 
** y compris les garanties de commercialisation ou d'adaptation dans un but spcifique.
** Reportez-vous  la Licence Publique Gnrale GNU pour plus de dtails.
**
** Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en mme temps que ce programme 
**
** bonus.c
**
** Gestions des bonus
**
**
**
*/

# include "bonus.h"

/*
** Definition des bonus
** Pour ajouter un bonus, modifier BONUS_NB dans game.h
** Modifier l'image
** Ajouter une ligne ici
** Les flags sont dfinits dans bonus.h
*/
struct s_bonusconf bonusconf[BONUS_NB] = {
	{"Nugget", 1, 100,
	 BONUS_IS_UNIQUE | BONUS_MAKE_WALL | BONUS_CAN_MAKE_BONUS},
	{"Gold", 5, 10, 0},
	{"Life", 10, 5, BONUS_GIVE_LIFE},
	{"Poison", 0, 5, BONUS_TAKE_LIFE},
	{"Ice", 5, 5, BONUS_CHANGE_ACCEL},
	{"Speed+", 5, 4, BONUS_SPEED_UP_1},
	{"Speed++", 10, 2, BONUS_SPEED_UP_2},
	{"Shield I", 5, 3, BONUS_INVIC_10},
	{"Shield II", 10, 1, BONUS_INVIC_20},
	{"Mithril", 15, 4}
};

/*
** Initilisation
*/
void InitBonus(struct s_bonus *bonus)
{
	unsigned int i;

	/* La liste chane est vide */
	bonus->first = NULL;
	bonus->last = NULL;

	/* Il y'a aucun bonus */
	for (i = 0; i < BONUS_NB; i++)
		bonus->bonuscount[i] = 0;
}

/*
** Libre la mmoire 
*/
void FreeBonus(struct s_bonus *bonus)
{
	struct s_1bonus *tmp;
	struct s_1bonus *pbonus;

	/* Virer les bonus, parcour de la liste chane */
	pbonus = bonus->first;
	while (pbonus) {
		tmp = pbonus;
		pbonus = pbonus->next;
		free(tmp);
	}
	bonus->first = NULL;
	bonus->last = NULL;
}

/*
** Calculs et Affichage de bonus
*/
void HandleBonus(struct s_bonus *bonus, struct s_walls *walls)
{
	unsigned int i, j, tmptype;
	struct s_1bonus *pbonus, *tmpptr;
	SDL_Rect src, dst;
	char tmp[256];

	src.w = BONUS_WIDTH;
	src.h = BONUS_HEIGHT;
	dst.w = src.w;
	dst.h = src.h;

	/* Parcour de la liste chane */
	pbonus = bonus->first;
	while (pbonus) {
		tmpptr = pbonus->next;
		src.x = pbonus->type * BONUS_WIDTH;
		src.y = 0;
		dst.x = pbonus->x;
		dst.y = pbonus->y;

		/* Affichage */
		SDL_BlitSurface(gameui.bonus, &src, SDL_GetVideoSurface(),
				&dst);

		/* Aucun joueur ne touche ce bonus ? */
		for (i = 0; i < options.playernb.value; i++) {
			/* Si la distance est plus petite que le rayon de la balle + le rayon du bonus */
			if (DSTBONUS
			    (balls[i].x, balls[i].y, pbonus->x,
			     pbonus->y) <= BALL_RADIUS + BONUS_RADIUS) {

				/* Ajout au score du joueur */
				balls[i].score +=
				    SCORE(bonusconf[pbonus->type].pts);
				sprintf(tmp, "%d",
					SCORE(bonusconf[pbonus->type].
					      pts));
				AddMessage(gameui.messages, tmp,
					   colors.sdl_blue, gameui.font,
					   pbonus->x, pbonus->y, 0, -1,
					   MESSAGE_BLITTING, 1000);

				/* Actions lies au bonus */
				if (bonusconf[pbonus->type].
				    flags & BONUS_MAKE_WALL) {
					AddWall(walls);
				}
				if (bonusconf[pbonus->type].
				    flags & BONUS_GIVE_LIFE) {
					balls[i].lifes++;
				}
				if (bonusconf[pbonus->type].
				    flags & BONUS_SPEED_UP_1) {
					balls[i].speed++;
				}
				if (bonusconf[pbonus->type].
				    flags & BONUS_SPEED_UP_2) {
					balls[i].speed += 2;
				}
				if (bonusconf[pbonus->type].
				    flags & BONUS_INVIC_10) {
					balls[i].invincibletotaltime = balls[i].invincibletime = 10 * 1000;	/* Rglages de l'invincibilite */
					balls[i].invinciblelast =
					    SDL_GetTicks();
				}
				if (bonusconf[pbonus->type].
				    flags & BONUS_INVIC_20) {
					balls[i].invincibletotaltime = balls[i].invincibletime = 20 * 1000;	/* Rglages de l'invincibilite */
					balls[i].invinciblelast =
					    SDL_GetTicks();
				}
				if (bonusconf[pbonus->type].
				    flags & BONUS_TAKE_LIFE) {
					balls[i].lifes--;
				}
				if (bonusconf[pbonus->type].
				    flags & BONUS_CHANGE_ACCEL) {
					for (j = 0;
					     j < options.playernb.value;
					     j++) {
						if (i != j)
							balls[j].accel +=
							    5;
					}
				}

				balls[i].bonuscount[pbonus->type]++;	/* on en a mang un de plus */
				PlaySDLWav(gameui.wavbonus);	/* Joue le son moisis */
				tmptype = pbonus->type;	/* avant de dtruire ce bonus, on stoque son type */
				DelBonus(bonus, pbonus);	/* On vire le vieux bonus */
				if (bonusconf[tmptype].
				    flags & BONUS_CAN_MAKE_BONUS)
					AddRandBonus(bonus);	/*Ajout d'un ou de plusieurs bonus */

				break;
			}
		}

		/* Suivant svp */
		pbonus = tmpptr;

	}
}

/*
** Ajoute un bonus alatoire 
*/
void AddRandBonus(struct s_bonus *bonus)
{
	unsigned int i;
	unsigned int bonusnb = 0;
	/* Tout les types de bonus */
	for (i = 0; i < BONUS_NB; i++) {
		/* Si le bonus n'est pas unique, ou qu'il est plus petit que 1 */
		/* Et que la proba est bonne */
		if (!((bonusconf[i].flags & BONUS_IS_UNIQUE)
		      && bonus->bonuscount[i] >= 1)
		    && myrand(0, 100) <= bonusconf[i].proba) {
			AddBonus(bonus, i,
				 myrand(0, SCREEN_WIDTH - BONUS_WIDTH),
				 myrand(0, SCREEN_HEIGHT - BONUS_HEIGHT));
			bonusnb++;
		}
		if (bonusnb >= 2)
			break;	//on met pas plus de deux bonus a la fois, trop simple sinon
	}
}

/*
** Ajoute un bonus de type spcifi  un endroit spcifi
*/
void
AddBonus(struct s_bonus *bonus, unsigned int type, unsigned int x,
	 unsigned int y)
{
	struct s_1bonus *new;
	new = malloc(sizeof(struct s_1bonus));

	new->prev = NULL;
	if (bonus->first == NULL)	/* Il n'y a pas de premier (chaine vide) */
		new->next = NULL;
	else			/* Sinon */
		new->next = bonus->first;


	if (bonus->last == NULL)	/* Il n'y pas de dernier (chaine vide) */
		bonus->last = new;
	else			/* Sinon */
		new->next->prev = new;

	bonus->first = new;	/* Le nouveau bonus mis au dbut */

	/* Rglages */
	new->x = x;
	new->y = y;
	new->type = type;

	/* Hop, y'en a un de plus de ce type l */
	bonus->bonuscount[type]++;
}

/*
** Supprime un bonus spcifi
*/
void DelBonus(struct s_bonus *bonus, struct s_1bonus *thebonus)
{
	if (thebonus->prev == NULL)	/* Si c'tait le premier */
		bonus->first = thebonus->next;
	else			/* Sinon */
		thebonus->prev->next = thebonus->next;

	if (thebonus->next == NULL)	/* Si c'tait le dernier */
		bonus->last = thebonus->prev;
	else			/* Sinon */
		thebonus->next->prev = thebonus->prev;

	/* Un bonus de se type l en moins */
	bonus->bonuscount[thebonus->type]--;

	free(thebonus);
}
