/*
** Copyright (C) 2005 CHARY Corentin <iksaif@gmail.com>
**
** Ce programme est libre, vous pouvez le redistribuer et/ou le modifier 
** selon les termes de la Licence Publique Gnrale GNU publie par la Free Software Foundation 
**
** Ce programme est distribu car potentiellement utile, mais SANS AUCUNE GARANTIE, ni explicite ni implicite, 
** y compris les garanties de commercialisation ou d'adaptation dans un but spcifique.
** Reportez-vous  la Licence Publique Gnrale GNU pour plus de dtails.
**
** Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en mme temps que ce programme 
**
** main.c
**
** Fichier principal
**
**
**
*/

# include "main.h"

/*
** Dclarations des variables globales utilises dans le jeu
*/
const struct s_colors colors = {
	{255, 0, 0, 0},
	{0, 0, 255, 0},
	{0, 255, 0, 0},
	{255, 255, 255, 0},
	{0, 0, 0, 0},
	{166, 166, 166, 0}
};

/* Options par defaut */
/* Modes de jeu */
const struct s_modes modes[OPTMAX_MODE + 1] = {
	{"Normal", 0},
	{"Arcade", MODE_ARCADE},
	{"Trace", MODE_TRACE},
	{"Arcade+Trace", MODE_ARCADE | MODE_TRACE}
	};

#ifdef _GP2X
/* Video */
const char *vmodes[] = { "Windowed", "Fullscreen" };
const struct s_vresols vresols[] = { {320, 240} };

struct s_options options = {
  {1, 1, 2, "Players", {NULL}},
  {0, 0, 2, "Computers", {NULL}},
  {0, 0, 3, "Mode", {"Normal", "Arcade", "Trace", "Arcade + Trace"}},
  {1, 1, 10, "Wall Size", {NULL}},
  {4, 4, 10, "Speed", {NULL}},
  {0, 0, 5, "Screen Size",
	 {"320x240", "800x600", "1024x768", "1152x864", "1280x960", "1280x1024"}},
  {0, 0, 1, "Video Mode", {"Windowed", "Fullscreen"}},
  0,
  0,
  {
    {GP2X_BUTTON_UP, GP2X_BUTTON_DOWN, GP2X_BUTTON_LEFT, GP2X_BUTTON_RIGHT},
    {GP2X_BUTTON_A, GP2X_BUTTON_Y, GP2X_BUTTON_Y, GP2X_BUTTON_X},
    {0, 0, 0, 0},
    {0, 0, 0, 0},
    {0, 0, 0, 0},
    {0, 0, 0, 0}
  },
  {
    "Player 1",
    "Player 2",
    "Player 3",
    "Player 4",
    "Player 5",
    "Player 6"
  }
};
#else
/* Video */
const char *vmodes[] = { "Windowed", "Fullscreen" };
const struct s_vresols vresols[OPTMAX_VRESOL + 1] =
{ {320, 240}, {800, 600}, {1024, 768}, {1152, 864}, {1280, 960}, {1280, 1024} };

struct s_options options = {
	{1, 1, 6, "Players", {NULL}},
	{0, 0, 6, "Computers", {NULL}},
	{0, 0, 3, "Mode", {"Normal", "Arcade", "Trace", "Arcade + Trace"}},
	{1, 1, 10, "Wall Size", {NULL}},
	{4, 4, 10, "Speed", {NULL}},
	{0, 0, 5, "Screen Size",
	 {"320x240", "800x600", "1024x768", "1152x864", "1280x960", "1280x1024"}},
	{0, 0, 1, "Video Mode", {"Windowed", "Fullscreen"}},
	0,
	0,
	{
	 {SDLK_UP, SDLK_DOWN, SDLK_LEFT, SDLK_RIGHT},
	 {SDLK_z, SDLK_s, SDLK_q, SDLK_d},
	 {SDLK_o, SDLK_l, SDLK_k, SDLK_m},
	 {SDLK_KP5, SDLK_KP2, SDLK_KP1, SDLK_KP3},
	 {SDLK_t, SDLK_g, SDLK_f, SDLK_h},
	 {SDLK_KP_DIVIDE, SDLK_KP8, SDLK_KP7, SDLK_KP9}
	},
	{
		"Player 1",
		"Player 2",
		"Player 3",
		"Player 4",
		"Player 5",
		"Player 6"
	}
};
#endif

/* pointeur vers la fentre */
SDL_Surface *screen;

int main(int argc, char **argv)
{
	LoadOptions();
	if (Init() > 0)
		return EXIT_FAILURE;

	/* Lance le menu */
	StartMenu();
	SaveOptions();
       
#ifdef _GP2X
	chdir("/usr/gp2x");
	execl("/usr/gp2x/gp2xmenu", "/usr/gp2x/gp2xmenu", NULL);
#endif
	return EXIT_SUCCESS;
}

/*
** Initialisation
** Retourne 0 si tout se passe bien, un chiffre plus grand sinon
*/
int Init()
{
	printf("Init SDL\n");

	/* Lorsqu'on quitte, on appelle la fonction "Free" */
	atexit(Free);

	/* 
	 ** On met le driver qui_va_bien, il semble de directx fonctionne mal, 
	 ** du moins sur les pc de la fac 
	 */
#ifdef WIN32
	putenv("SDL_VIDEODRIVER=windib");
#endif

	/* On Initialise SDL (video et audio) */
	if (SDL_Init(SDL_INIT_VIDEO | SDL_INIT_AUDIO) == -1) {
		fprintf(stderr, "Can't Init SDL: %s\n", SDL_GetError());
		return 1;
	}

	/* Initialisation de la fentre */
	int flags = SDL_DOUBLEBUF;
	if(options.vmode.value) {
		flags |= SDL_FULLSCREEN;
	}
	SetVideoMode(SCREEN_WIDTH, SCREEN_HEIGHT, 16, flags);

	if (screen == NULL) {
		perror("Can't set video mode.\n");
		return 1;
	}

	SDL_EnableUNICODE(1);
	/* Initialisation de SDL_ttf */
	if (TTF_Init() == -1) {
		fprintf(stderr, "Can't init SDL_TTF: %s\n",
			TTF_GetError());
		return 1;
	}



	/* Initialisation de SDL_Mixer */
	if (Mix_OpenAudio(MIX_DEFAULT_FREQUENCY, AUDIO_S16, MIX_DEFAULT_CHANNELS, 128) != 0) {
		fprintf(stderr, "Can't init SDL_Mixer: %s\n",
			Mix_GetError());
		return 1;
	}
	Mix_VolumeMusic(20);


	/* Rglages des rptitions du clavier */
	SDL_EnableKeyRepeat(100, 100);

	return 0;
}

/*
** Fonction permettant de fermer proprement le programme en librant tout ce qu'il faut.
*/
void Free()
{
	Mix_CloseAudio();
	TTF_Quit();
	SDL_Quit();
}
