/*
** Copyright (C) 2005 CHARY Corentin <iksaif@gmail.com>
**
** Ce programme est libre, vous pouvez le redistribuer et/ou le modifier 
** selon les termes de la Licence Publique Gnrale GNU publie par la Free Software Foundation 
**
** Ce programme est distribu car potentiellement utile, mais SANS AUCUNE GARANTIE, ni explicite ni implicite, 
** y compris les garanties de commercialisation ou d'adaptation dans un but spcifique.
** Reportez-vous  la Licence Publique Gnrale GNU pour plus de dtails.
**
** Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en mme temps que ce programme 
**
** utils.c
**
** Fonctions utiles un peu partout
**
**
**
*/

# include "utils.h"

/*
** Fonction permettant de limiter le nombre de fps pour pas prendre tout le cpu
*/
void limitFps()
{
	static int cur_time = 1;
	static int last_time = 0;
	static int frames_to_wait = FPS;	/* Frames restant avant de recalculer le nombre de frames/sec */
	const unsigned int min_loop_length = 1000 / FPS;	/* Temps minimal d'execution d'une boucle */
	static unsigned int sum_loop_length = 1000 / FPS;	/* Temps requis pour afficher FPS frames */

	cur_time = SDL_GetTicks();
	if (last_time > cur_time) {	/* SDL_GetTicks boucle tous les 47 jours */
		last_time = 0;
		cur_time = 1;	/* Il ne faut pas que loop_lengt==0 */
	}
	unsigned int loop_length = cur_time - last_time;	/* Temps reel d'execution d'une boucle */


	if (loop_length < min_loop_length) {
		SDL_Delay(min_loop_length - loop_length);
		sum_loop_length += min_loop_length - loop_length;	/* Vu qu'il y'a un delay, on rajoute le temps au total. */
	}

	if (frames_to_wait) {
		sum_loop_length += loop_length;
		--frames_to_wait;
	} else {
		sum_loop_length += loop_length;
		nb_fps = FPS * 1000 / sum_loop_length;
		sum_loop_length = loop_length;
		frames_to_wait = FPS;
	}
	last_time = cur_time;
}

/*
** Permet de charger une image, vite de rpter trop souvent le mme code.
*/
SDL_Surface *LoadSDLImage(const char *filename, unsigned int alpha)
{
	SDL_Surface *tmp;
	SDL_Surface *ret;

	if ((tmp = IMG_Load(filename)) == NULL) {	/* On charge l'image avec SDL_image */
		fprintf(stderr, "%s\n", SDL_GetError());
		return NULL;
	}

	if ((ret = (alpha) ? SDL_DisplayFormatAlpha(tmp)
	     : SDL_DisplayFormat(tmp)) == NULL) {	/* On utilise la transparence ou pas ... */
		fprintf(stderr, "%s\n", SDL_GetError());
		SDL_FreeSurface(tmp);
		return NULL;
	}

	SDL_FreeSurface(tmp);

	return ret;
}

/*
** Permet de charger une police
*/
TTF_Font *LoadSDLFont(const char *filename, unsigned int size)
{
	TTF_Font *tmp;

	tmp = TTF_OpenFont(filename, size);	/* Chargement de la police */
	if (!tmp) {
		fprintf(stderr, "TTF_OpenFont: %s\n", TTF_GetError());
		return NULL;
	}

	return tmp;
}

/*
** Gnere un entier pseudo alatoire entre a et b
*/
unsigned int myrand(unsigned int a, unsigned int b)
{
	static unsigned int init = 0;
	if (!init) {
		srand(time(0));
		init = 1;
	}
	return (unsigned int) a + ((float) rand() * (b - a) / RAND_MAX);
}

/*
** Compte le nombre de 1 d'un nombre
*/
unsigned int count_ones(unsigned int n)
{
	unsigned int i, nb = 0;

	for (i = 0; i < sizeof(unsigned int) * 8; ++i)
		if (n & (1 << i))
			++nb;

	return nb;
}

/*
** Dessine une barre sur screen positionne avec dst
*/
void
DrawGraph(SDL_Surface * surf, SDL_Rect dst, unsigned int value,
	  unsigned int valuemax)
{
	if (valuemax < value || !valuemax)
		return;

	SDL_Rect tmp;
	tmp = dst;

	SDL_FillRect(surf, &tmp, SDL_MapRGB(surf->format, 166, 166, 166));
	tmp.x++;
	tmp.y++;
	tmp.h -= 2;
	tmp.w = dst.w * value / valuemax;
	SDL_FillRect(surf, &tmp, SDL_MapRGB(surf->format, 60, 166, 60));

	if (tmp.w <= dst.w - 2) {
		tmp.x = tmp.w + dst.x + 1;
		tmp.w = dst.w - tmp.w - 2;
		SDL_FillRect(surf, &tmp,
			     SDL_MapRGB(surf->format, 255, 0, 0));
	}
}

/*
** Fonction gnrique pour afficher du texte n'importe o.
** Le paramtre dst est ajust en fonction de la largeur et de la longueur du texte
*/
void
PrintText(SDL_Surface * surf, char *text, SDL_Rect * dst,
	  TTF_Font * font, SDL_Color color)
{
	SDL_Surface *text_surf;

	text_surf = TTF_RenderText_Blended(font, text, color);
	SDL_BlitSurface(text_surf, NULL, SDL_GetVideoSurface(), dst);
	SDL_FreeSurface(text_surf);
}

/*
** Avoir des jolis flags compatibles
*/
unsigned int get_flags(unsigned int flags)
{

  flags = SDL_SWSURFACE;
	return flags;
}

/*
** Changer le mode video
*/
unsigned int
SetVideoMode(unsigned int x, unsigned int y,
	     unsigned int bits_per_pixel, unsigned int flags)
{
	flags = get_flags(flags);	/* (P)Rparation des flags */

	unsigned int res = SDL_VideoModeOK(x, y, 16, flags);

	if (res == 0) {		/* Bha non */
		fprintf(stderr, "Can't change video mode to %dx%d@%d\n", x,
			y, bits_per_pixel);
		return 0;
	}

	/* Changement du mode video */
	screen = SDL_SetVideoMode(x, y, 16, flags);

	if (screen != NULL) {
		return bits_per_pixel;
	} else
		return 0;
}


/* Afficher des infos sur la video */
void GetVideoInfo(void)
{
	const SDL_VideoInfo *infos;
	infos = SDL_GetVideoInfo();
	
	printf("hw_available: %d\n",infos->hw_available);
	printf("wm_available: %d\n",infos->wm_available);
	printf("hw to hw blits: %d\n",infos->blit_hw);
	printf("hw to hw colorkey blits: %d\n",infos->blit_hw_CC);
	printf("hw to hw alpha blits: %d\n",infos->blit_hw_A);
	printf("sw to hw blits: %d\n",infos->blit_sw);
	printf("sw to hw colorkey blits: %d\n",infos->blit_sw_CC);
	printf("sw to hw alpha blits: %d\n",infos->blit_sw_A);
	printf("color fills: %d\n",infos->blit_fill);
	printf("video memory: %d ko\n",infos->video_mem);
	printf("Pixel format : %d %d\n",infos->vfmt->BitsPerPixel,infos->vfmt->BytesPerPixel);
}
