/*
** Copyright (C) 2005 CHARY Corentin <iksaif@gmail.com>
**
** Ce programme est libre, vous pouvez le redistribuer et/ou le modifier 
** selon les termes de la Licence Publique Gnrale GNU publie par la Free Software Foundation 
**
** Ce programme est distribu car potentiellement utile, mais SANS AUCUNE GARANTIE, ni explicite ni implicite, 
** y compris les garanties de commercialisation ou d'adaptation dans un but spcifique.
** Reportez-vous  la Licence Publique Gnrale GNU pour plus de dtails.
**
** Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en mme temps que ce programme 
**
** walls.c
**
** Gestions des Murs
**
**
**
*/

# include "walls.h"

/*
** Initialisation
*/
void InitWalls(struct s_walls *walls)
{
	walls->walls = malloc(WALLS_BUFFER * sizeof(struct s_wall));
	walls->wallsnb = 0;
}

/*
** Libration de la mmoire
*/
void FreeWalls(struct s_walls *walls)
{
	free(walls->walls);
	walls->walls = NULL;
}

/*
** Bouge les murs
** Le param bonus sert aux collisions murs-bonus, mais c'est pas encore cod
*/
void MoveWalls(struct s_walls *walls, struct s_bonus *bonus)
{
	unsigned int i, j;
	double speed = (double) options.speed.value / 3.0f;

	/* Parcour des murs */
	for (i = 0; i < walls->wallsnb; i++) {
		if (walls->walls[i].type == WALLS_VERT) {	/* Pour les murs a dplacement Vertical */
			if (walls->walls[i].y < WALLS_BORDER_DECAL) {	/* Si le Mur est trop proche du haut, on l'arrete et on le fait repartir dans l'autre sens */
				walls->walls[i].dy = speed / WALLS_ACCEL;
			} else if (walls->walls[i].y > SCREEN_HEIGHT - WALLS_BORDER_DECAL - options.wallsize.value * WALLS_SIZE_COEF) {	/* Trop bas */
				walls->walls[i].dy = -speed / WALLS_ACCEL;
			} else if (walls->walls[i].dy > 0 && walls->walls[i].dy < speed) {	/* En train d'accelerer (vers le bas) */
				walls->walls[i].dy += speed / WALLS_ACCEL;
			} else if (walls->walls[i].dy < 0 && -walls->walls[i].dy < speed) {	/* Vers le haut */
				walls->walls[i].dy -= speed / WALLS_ACCEL;
			}

			walls->walls[i].y += walls->walls[i].dy;
		} else {	/* Dplacement horizontal */

			if (walls->walls[i].x < WALLS_BORDER_DECAL) {	/* Trop a gauche */
				walls->walls[i].dx = speed / WALLS_ACCEL;
			} else if (walls->walls[i].x > SCREEN_WIDTH - WALLS_BORDER_DECAL - options.wallsize.value * WALLS_SIZE_COEF) {	/* Trop a droite */
				walls->walls[i].dx = -speed / WALLS_ACCEL;
			} else if (walls->walls[i].dx > 0 && walls->walls[i].dx < speed) {	/* Acceleration vers la droite */
				walls->walls[i].dx += speed / WALLS_ACCEL;
			} else if (walls->walls[i].dx < 0 && -walls->walls[i].dx < speed) {	/* Vers la droite */
				walls->walls[i].dx -= speed / WALLS_ACCEL;
			}
			walls->walls[i].x += walls->walls[i].dx;
		}

		/* Verification pour les collisions */
		for (j = 0; j < options.playernb.value; ++j) {
			if (balls[j].lifes <= 0
			    || balls[j].invincibletime > 0)
				continue;	/* Si la balle est invincible, inutile de continuer */

			if (walls->walls[i].type == WALLS_VERT) {	/* Murs verticaux */
				if (balls[j].x + BALL_WIDTH >= walls->walls[i].x && balls[j].x <= walls->walls[i].x + WALLS_WIDTH && balls[j].y + BALL_HEIGHT >= walls->walls[i].y && balls[j].y <= walls->walls[i].y + options.wallsize.value * WALLS_SIZE_COEF) {	/* Collision ! */
					balls[j].lifes--;
					balls[j].invincibletotaltime = balls[j].invincibletime = WALLS_INVIC_TIME;	/* Rglages de l'invincibilite */
					balls[j].invinciblelast =
					    SDL_GetTicks();

					if (modes[options.mode.value].flags & MODE_ARCADE)	/* Si Mode Arcade, repartir dans l'autre sens */
						walls->walls[i].dy =
						    -1 *
						    walls->walls[i].dy;
					AddMessage(gameui.messages,
						   "BaAaAAm",
						   colors.sdl_blue,
						   gameui.font, balls[j].x,
						   balls[j].y, 0, 0,
						   MESSAGE_FADING, 1000);
				}

			} else {	/* Hozizontaux */

				if (balls[j].x + BALL_WIDTH >=
				    walls->walls[i].x
				    && balls[j].x <=
				    walls->walls[i].x +
				    options.wallsize.value *
				    WALLS_SIZE_COEF
				    && balls[j].y + BALL_HEIGHT >=
				    walls->walls[i].y
				    && balls[j].y <=
				    walls->walls[i].y + WALLS_WIDTH) {
					balls[j].lifes--;
					balls[j].invincibletotaltime = balls[j].invincibletime = WALLS_INVIC_TIME;	/* Rglages de l'invincibilite */
					balls[j].invinciblelast =
					    SDL_GetTicks();
					if (modes[options.mode.value].flags & MODE_ARCADE)	/* Si Mode Arcade, repartir dans l'autre sens */
						walls->walls[i].dx =
						    -1 *
						    walls->walls[i].dx;
					AddMessage(gameui.messages,
						   "BaAaAAm",
						   colors.sdl_blue,
						   gameui.font, balls[j].x,
						   balls[j].y, 0, 0,
						   MESSAGE_FADING, 1000);
				}
			}
		}

	}
}

/*
** Gestion des murs
** Dplacements, puis affichage
*/
void HandleWalls(struct s_walls *walls, struct s_bonus *bonus)
{
	unsigned int i;
	SDL_Rect src, dst;

	MoveWalls(walls, bonus);

	for (i = 0; i < walls->wallsnb; i++) {
		if (modes[options.mode.value].flags & MODE_TRACE) {	/* On affiche la trajectoire des murs */
			if (walls->walls[i].type == WALLS_VERT) {
				dst.x = walls->walls[i].x;
				dst.y = 0;
				dst.w = WALLS_WIDTH;
				dst.h = SCREEN_HEIGHT;
			} else {
				dst.x = 0;
				dst.y = walls->walls[i].y;
				dst.w = SCREEN_WIDTH;
				dst.h = WALLS_WIDTH;
			}

			SDL_FillRect(SDL_GetVideoSurface(), &dst,
				     SDL_MapRGB(screen->format, 200, 200,
						200));
		}

		if (walls->walls[i].type == WALLS_VERT) {
			src.x = 0;
			src.y = 5;
			src.w = WALLS_WIDTH;
			src.h = options.wallsize.value * WALLS_SIZE_COEF;
		} else {
			src.x = 5;
			src.y = 0;
			src.w = options.wallsize.value * WALLS_SIZE_COEF;
			src.h = WALLS_WIDTH;
		}

		dst.x = walls->walls[i].x;
		dst.y = walls->walls[i].y;
		dst.w = src.w;
		dst.h = src.h;

		SDL_BlitSurface(gameui.walls, &src, SDL_GetVideoSurface(),
				&dst);
	}

}

/*
** Ajout d'un mur
*/
void AddWall(struct s_walls *walls)
{
	double speed = (double) options.speed.value / 3.0f;

	if (walls->wallsnb > 0 && walls->wallsnb % WALLS_BUFFER == 0) {	/* Si le tableau est plein */
		walls->walls = realloc(walls->walls, (walls->wallsnb + WALLS_BUFFER) * sizeof(struct s_wall));	/* On augmente sa taille */
	}

	/* Ajout du mur en fonction de trucs alatoires */
	walls->walls[walls->wallsnb].type =
	    myrand(WALLS_VERT, WALLS_HORI + 1);
	if (walls->walls[walls->wallsnb].type == WALLS_VERT) {
		walls->walls[walls->wallsnb].x =
		    myrand(WALLS_BORDER_DECAL,
			   SCREEN_WIDTH - WALLS_BORDER_DECAL -
			   WALLS_WIDTH);

		walls->walls[walls->wallsnb].y =
		    myrand(WALLS_BORDER_DECAL,
			   SCREEN_HEIGHT - WALLS_BORDER_DECAL -
			   options.wallsize.value * WALLS_SIZE_COEF);

		walls->walls[walls->wallsnb].dx = 0;

		walls->walls[walls->wallsnb].dy = speed / WALLS_ACCEL;
	} else {
		walls->walls[walls->wallsnb].x =
		    myrand(WALLS_BORDER_DECAL,
			   SCREEN_WIDTH - WALLS_BORDER_DECAL -
			   options.wallsize.value * WALLS_SIZE_COEF);

		walls->walls[walls->wallsnb].y =
		    myrand(WALLS_BORDER_DECAL,
			   SCREEN_HEIGHT - WALLS_BORDER_DECAL -
			   WALLS_WIDTH);

		walls->walls[walls->wallsnb].dx = speed / WALLS_ACCEL;

		walls->walls[walls->wallsnb].dy = 0;
	}


	walls->wallsnb++;
}
